local VERSION = tonumber(("$Revision: 44067 $"):match("%d+"))

local Parrot = Parrot
local Parrot_ScrollAreas = Parrot:NewModule("ScrollAreas", "AceEvent-2.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-18 03:24:34 -0400 (Wed, 18 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot_ScrollAreas")

local scrollAreas

local validate = {}

Parrot_ScrollAreas.db = Parrot:AcquireDBNamespace("ScrollAreas")

function Parrot_ScrollAreas:OnEnable()
	if not self.db.profile.areas then
		self.db.profile.areas = {
			["Notification"] = {
				align = "CENTER",
				animationStyle = "Straight",
				stickyAnimationStyle = "Pow",
				reverse = false,
				stickyReverse = false,
				size = 150,
				xOffset = 0,
				yOffset = 175,
			},
			["Incoming"] = {
				align = "RIGHT",
				animationStyle = "Left parabola",
				stickyAnimationStyle = "Pow",
				reverse = true,
				stickyReverse = true,
				size = 260,
				xOffset = -60,
				yOffset = -30,
			},
			["Outgoing"] = {
				align = "LEFT",
				animationStyle = "Right parabola",
				stickyAnimationStyle = "Pow",
				reverse = true,
				stickyReverse = true,
				size = 260,
				xOffset = 60,
				yOffset = -30,
			},
		}
	end	
	scrollAreas = self.db.profile.areas
	for k, v in pairs(scrollAreas) do
		if k == "Notification" or k == "Incoming" or k == "Outgoing" then
			validate[k] = L[k]
		else
			validate[k] = k
		end
	end
end

function Parrot_ScrollAreas:HasScrollArea(name)
	return not not scrollAreas[name]
end

function Parrot_ScrollAreas:GetScrollArea(name)
	return scrollAreas[name]
end

function Parrot_ScrollAreas:GetRandomScrollArea()
	local i = 0
	for k, v in pairs(scrollAreas) do
		i = i + 1
	end
	local num = math.random(1, i)
	i = 0
	for k, v in pairs(scrollAreas) do
		i = i + 1
		if i == num then
			return k, v
		end
	end
	return
end

function Parrot_ScrollAreas:GetScrollAreasValidate(name)
	return validate
end

function Parrot_ScrollAreas:OnOptionsCreate()
	local configMode = false
	
	local test
	local function getName(name)
		if name == "Notification" or name == "Incoming" or name == "Outgoing" then
			name = L[name]
		end
		return name
	end
	local function setName(old, new)
		if old == new or scrollAreas[new] then
			return
		end
		local v = scrollAreas[old]
		scrollAreas[old] = nil
		scrollAreas[new] = v
		local opt = Parrot.options.args.scrollAreas.args[tostring(v)]
		local name = new
		if name == "Notification" or name == "Incoming" or name == "Outgoing" then
			name = L[name]
		end
		validate[old] = nil
		validate[new] = name
		opt.name = name
		opt.args.name.passValue = new
		opt.args.font.args.fontface.passValue = new
		opt.args.font.args.fontSizeInherit.passValue = new
		opt.args.font.args.fontSize.passValue = new
		opt.args.font.args.fontOutline.passValue = new
	end
	local alphabet = {
		"Alpha", "Bravo", "Charlie", "Delta", "Echo", "Foxtrot", "Golf",
		"Hotel", "India", "Juliet", "Kilo", "Mike", "November", "Oscar",
		"Papa", "Quebec", "Romeo", "Sierra", "Tango", "Uniform", "Victor",
		"Whiskey", "X-ray", "Yankee", "Zulu",
	}
	local currentAlphabet = 1
	local currentColor = 0
	function test(passValue)
		local kind, k = (";"):split(passValue, 2)
		local h = currentColor / 60
		local i = math.floor(h)
		local f = h - i
		local color
		if i == 0 then
			color = ("%02x%02x%02x"):format(255, f*255, 0)
		elseif i == 1 then
			color = ("%02x%02x%02x"):format((1 - f)*255, 255, 0)
		elseif i == 2 then
			color = ("%02x%02x%02x"):format(0, 255, f*255)
		elseif i == 3 then
			color = ("%02x%02x%02x"):format(0, (1 - f)*255, 255)
		elseif i == 4 then
			color = ("%02x%02x%02x"):format(f*255, 0, 255)
		else -- 5
			color = ("%02x%02x%02x"):format(255, 0, (1 - f)*255)
		end
		Parrot:GetModule("Display"):ShowMessage(alphabet[currentAlphabet], k, kind == "sticky", color)
		currentAlphabet = (currentAlphabet%(#alphabet)) + 1
		currentColor = (currentColor + 10) % 360
	end
	local function getFontFace(passValue)
		local kind, k = (";"):split(passValue, 2)
		local font = scrollAreas[k][kind == "normal" and "font" or "stickyFont"]
		if font == nil then
			return L["Inherit"]
		else
			return font
		end
	end
	local function setFontFace(passValue, value)
		local kind, k = (";"):split(passValue, 2)
		if value == L["Inherit"] then
			value = nil
		end
		scrollAreas[k][kind == "normal" and "font" or "stickyFont"] = value
		if not configMode then
			test(passValue)
		end
	end
	local function getFontSize(passValue)
		local kind, k = (";"):split(passValue, 2)
		return scrollAreas[k][kind == "normal" and "fontSize" or "stickyFontSize"]
	end
	local function setFontSize(passValue, value)
		local kind, k = (";"):split(passValue, 2)
		scrollAreas[k][kind == "normal" and "fontSize" or "stickyFontSize"] = value
		if not configMode then
			test(passValue)
		end
	end
	local function getFontSizeInherit(passValue)
		local kind, k = (";"):split(passValue, 2)
		return scrollAreas[k][kind == "normal" and "fontSize" or "stickyFontSize"] == nil
	end
	local function setFontSizeInherit(passValue, value)
		local kind, k = (";"):split(passValue, 2)
		if value then
			scrollAreas[k][kind == "normal" and "fontSize" or "stickyFontSize"] = nil
		else
			scrollAreas[k][kind == "normal" and "fontSize" or "stickyFontSize"] = 18
		end
		if not configMode then
			test(passValue)
		end
	end
	local function getFontOutline(passValue)
		local kind, k = (";"):split(passValue, 2)
		local outline = scrollAreas[k][kind == "normal" and "fontOutline" or "stickyFontOutline"]
		if outline == nil then
			return L["Inherit"]
		else
			return outline
		end
	end
	local function setFontOutline(passValue, value)
		local kind, k = (";"):split(passValue, 2)
		if value == L["Inherit"] then
			value = nil
		end
		scrollAreas[k][kind == "normal" and "fontOutline" or "stickyFontOutline"] = value
		if not configMode then
			test(passValue)
		end
	end
	local fontOutlineValidate = {
		NONE = L["None"],
		OUTLINE = L["Thin"],
		THICKOUTLINE = L["Thick"],
		[L["Inherit"]] = L["Inherit"],
	}
	local function getReverse(passValue)
		local kind, k = (";"):split(passValue, 2)
		return scrollAreas[k][kind == "normal" and "reverse" or "stickyReverse"]
	end
	local function setReverse(passValue, value)
		local kind, k = (";"):split(passValue, 2)
		scrollAreas[k][kind == "normal" and "reverse" or "stickyReverse"] = value
		if not configMode then
			test(passValue)
		end
	end
	local function getAnimationStyle(passValue)
		local kind, k = (";"):split(passValue, 2)
		return scrollAreas[k][kind == "normal" and "animationStyle" or "stickyAnimationStyle"]
	end
	local function setAnimationStyle(passValue, value)
		local kind, k = (";"):split(passValue, 2)
		scrollAreas[k][kind == "normal" and "animationStyle" or "stickyAnimationStyle"] = value
		if not configMode then
			test(passValue)
		end
	end
	local function getAlign(k)
		return scrollAreas[k].align
	end
	local function setAlign(k, value)
		scrollAreas[k].align = value
		if not configMode then
			test("normal;" .. k)
			test("normal;" .. k)
			test("normal;" .. k)
			test("sticky;" .. k)
			test("sticky;" .. k)
			test("sticky;" .. k)
		end
	end
	local function getSize(k)
		return scrollAreas[k].size
	end
	local function setSize(k, value)
		scrollAreas[k].size = value
		if not configMode then
			test("normal;" .. k)
			test("sticky;" .. k)
		end
	end
	local alignValidate = {
		LEFT = L["Left"],
		CENTER = L["Center"],
		RIGHT = L["Right"],
	}
	local function remove(k)
		Parrot.options.args.scrollAreas.args[tostring(scrollAreas[k])] = nil
		scrollAreas[k] = nil
		validate[k] = nil
	end
	local function disableRemove(k)
		return not next(scrollAreas, next(scrollAreas))
	end
	local function makeOption(k)
		local v = scrollAreas[k]
		local name = k
		if name == "Notification" or name == "Incoming" or name == "Outgoing" then
			name = L[name]
		end
		local opt = {
			type = 'group',
			name = name,
			desc = L["Options for this scroll area."],
			args = {
				name = {
					type = 'text',
					name = L["Name"],
					desc = L["Name of the scroll area."],
					get = getName,
					set = setName,
					usage = L["<Name>"],
					validate = nameValidate,
					passValue = k,
					order = 1,
				},
				remove = {
					type = 'execute',
					name = L["Remove"],
					desc = L["Remove this scroll area."],
					func = remove,
					disabled = disableRemove,
					passValue = k,
					order = -1,
					confirm = true,
				},
				test = {
					type = 'group',
					name = L["Test"],
					desc = L["Send a test message through this scroll area."],
					args = {
						normal = {
							type = 'execute',
							name = L["Normal"],
							desc = L["Send a normal test message."],
							func = test,
							passValue = "normal;" ..k,
						},
						sticky = {
							type = 'execute',
							name = L["Sticky"],
							desc = L["Send a sticky test message."],
							func = test,
							passValue = "sticky;" ..k,
						},
					},
					disabled = function() return configMode end
				},
				reverse = {
					type = 'group',
					name = L["Reverse direction"],
					desc = L["Whether to reverse the direction of the text scroll."],
					args = {
						normal = {
							type = 'toggle',
							name = L["Normal"],
							desc = L["Reverse for normal texts."],
							get = getReverse,
							set = setReverse,
							passValue = "normal;" ..k,
						},
						sticky = {
							type = 'toggle',
							name = L["Sticky"],
							desc = L["Reverse for sticky texts."],
							get = getReverse,
							set = setReverse,
							passValue = "sticky;" ..k,
						},
					}
				},
				animationStyle = {
					type = 'group',
					name = L["Animation style"],
					desc = L["Which animation style to use."],
					args = {
						normal = {
							type = 'text',
							name = L["Normal"],
							desc = L["Animation style for normal texts."],
							get = getAnimationStyle,
							set = setAnimationStyle,
							validate = Parrot:GetModule("AnimationStyles"):GetAnimationStylesValidate(),
							passValue = "normal;" .. k,
						},
						sticky = {
							type = 'text',
							name = L["Sticky"],
							desc = L["Animation style for sticky texts."],
							get = getAnimationStyle,
							set = setAnimationStyle,
							validate = Parrot:GetModule("AnimationStyles"):GetAnimationStylesValidate(),
							passValue = "sticky;" .. k,
						},
					}
				},
				align = {
					type = 'text',
					name = L["Align"],
					desc = L["Which position to align the text to."],
					get = getAlign,
					set = setAlign,
					validate = alignValidate,
					passValue = k,
				},
				size = {
					type = 'range',
					name = L["Size"],
					desc = L["How large of an area to scroll."],
					get = getSize,
					set = setSize,
					min = 50,
					max = 800,
					step = 1,
					bigStep = 10,
					passValue = k,
				},
				move = nil and {
					type = 'execute',
					name = L["Move"],
					desc = L["Move the scroll area to a more desireable location."],
					func = move,
					passValue = k,
				},
				font = {
					type = 'group',
					name = L["Custom font"],
					desc = L["Custom font"],
					args = {
						fontface = {
							type = 'text',
							name = L["Normal font face"],
							desc = L["Normal font face"],
							validate = Parrot.inheritFontValidate,
							get = getFontFace,
							set = setFontFace,
							passValue = "normal;" .. k,
							order = 1,
						},
						fontSizeInherit = {
							type = 'toggle',
							name = L["Normal inherit font size"],
							desc = L["Normal inherit font size"],
							get = getFontSizeInherit,
							set = setFontSizeInherit,
							passValue = "normal;" .. k,
							order = 2,
						},
						fontSize = {
							type = 'range',
							name = L["Normal font size"],
							desc = L["Normal font size"],
							min = 12,
							max = 30,
							step = 1,
							get = getFontSize,
							set = setFontSize,
							disabled = getFontSizeInherit,
							passValue = "normal;" .. k,
							order = 3,
						},
						fontOutline = {
							type = 'text',
							name = L["Normal font outline"],
							desc = L["Normal font outline"],
							get = getFontOutline,
							set = setFontOutline,
							validate = fontOutlineValidate,
							passValue = "normal;" .. k,
							order = 4,
						},
						stickyfontface = {
							type = 'text',
							name = L["Sticky font face"],
							desc = L["Sticky font face"],
							validate = Parrot.inheritFontValidate,
							get = getFontFace,
							set = setFontFace,
							passValue = "sticky;" .. k,
							order = 5,
						},
						stickyfontSizeInherit = {
							type = 'toggle',
							name = L["Sticky inherit font size"],
							desc = L["Sticky inherit font size"],
							get = getFontSizeInherit,
							set = setFontSizeInherit,
							passValue = "sticky;" .. k,
							order = 6,
						},
						stickyfontSize = {
							type = 'range',
							name = L["Sticky font size"],
							desc = L["Sticky font size"],
							min = 12,
							max = 30,
							step = 1,
							get = getFontSize,
							set = setFontSize,
							disabled = getFontSizeInherit,
							passValue = "sticky;" .. k,
							order = 7,
						},
						stickyfontOutline = {
							type = 'text',
							name = L["Sticky font outline"],
							desc = L["Sticky font outline"],
							get = getFontOutline,
							set = setFontOutline,
							validate = fontOutlineValidate,
							passValue = "sticky;" .. k,
							order = 8,
						},
					}
				}
			},
			order = -100,
		}
		Parrot.options.args.scrollAreas.args[tostring(v)] = opt
	end
	
	local offsetBoxes
	local function hideAllOffsetBoxes()
		if not offsetBoxes then
			return
		end
		for k,v in pairs(offsetBoxes) do
			v:Hide()
		end
		Parrot_ScrollAreas:CancelScheduledEvent("Parrot_ScrollAreas-configModeMessages")
	end
	local function showOffsetBox(k)
		if not offsetBoxes then
			offsetBoxes = {}
		end
		local offsetBox = offsetBoxes[k]
		if not offsetBox then
			offsetBox = CreateFrame("Button", "Parrot_ScrollAreas_OffsetBox_" .. k, UIParent)
			offsetBoxes[k] = offsetBox
			offsetBox:SetWidth(300)
			offsetBox:SetHeight(100)
			offsetBox:SetFrameStrata("MEDIUM")
			
			local bg = offsetBox:CreateTexture("Parrot_ScrollAreas_OffsetBox_" .. k .. "_Background", "BACKGROUND")
			bg:SetTexture(0.7, 0.4, 0, 0.5) -- orange
			bg:SetAllPoints(offsetBox)
			
			local text = offsetBox:CreateFontString("Parrot_ScrollAreas_Offset_" .. k .. "_BoxText", "ARTWORK", "GameFontHighlight")
			offsetBox.text = text
			text:SetPoint("CENTER")
			local name = k
			if name == "Notification" or name == "Incoming" or name == "Outgoing" then
				name = L[name]
			end
			text:SetText(L["Scroll area: %s\nClick and drag to the position you want."]:format(name))
			
			offsetBox:SetScript("OnDragStart", function(this)
				this:StartMoving()
				this.moving = true
			end)
			
			offsetBox:SetScript("OnDragStop", function(this)
				this.moving = nil
				this:StopMovingOrSizing()
			end)
			
			offsetBox:SetScript("OnUpdate", function(this)
				if this.moving then
					local x, y = this:GetCenter()
					x = x - GetScreenWidth()/2
					y = y - GetScreenHeight()/2
					scrollAreas[k].xOffset = x
					scrollAreas[k].yOffset = y
				end
			end)
			
			offsetBox:SetMovable(true)
			offsetBox:RegisterForDrag("LeftButton")
			offsetBox:Hide()
		end

		offsetBox:Show()
		
		offsetBox:ClearAllPoints()
		local offsetX, offsetY = scrollAreas[k].xOffset, scrollAreas[k].yOffset
		offsetBox:SetPoint("CENTER", UIParent, "CENTER", offsetX, offsetY)
		
		offsetBox:SetClampedToScreen(true)
	end
	
	local num = 0
	local function configModeMessages()
		num = num%2 + 1
		for k in pairs(scrollAreas) do
			test("normal;" .. k)
		end
		if num == 2 then
			for k in pairs(scrollAreas) do
				test("sticky;" .. k)
			end
		end
	end
	
	local function setConfigMode(value)
		configMode = value
		if not value then
			hideAllOffsetBoxes()
		else
			for k in pairs(scrollAreas) do
				showOffsetBox(k)
			end
			Parrot_ScrollAreas:ScheduleRepeatingEvent("Parrot_ScrollAreas-configModeMessages", configModeMessages, 1)
			configModeMessages()
		end
	end
	Parrot.options.args.scrollAreas = {
		type = 'group',
		name = L["Scroll areas"],
		desc = L["Options regarding scroll areas."],
		args = {
			config = {
				type = 'toggle',
				name = L["Configuration mode"],
				desc = L["Enter configuration mode, allowing you to move around the scroll areas and see them in action."],
				get = function()
					return configMode
				end,
				set = setConfigMode
			},
			new = {
				type = 'execute',
				name = L["New scroll area"],
				desc = L["Add a new scroll area."],
				func = function()
					scrollAreas[L["New scroll area"]] = {
						align = "CENTER",
						animationStyle = "Straight",
						stickyAnimationStyle = "Pow",
						reverse = false,
						stickyReverse = false,
						size = 150,
						xOffset = 0,
						yOffset = 0,
					}
					makeOption(L["New scroll area"])
				end,
				disabled = function()
					return scrollAreas[L["New scroll area"]]
				end
			}
		},
	}
	for k in pairs(scrollAreas) do
		makeOption(k)
	end
end